/* paramfield.c */

/*
 *  Copyright (c) 1992, Research Systems Inc.  All rights reserved.
 *   Reproduction by any means whatsoever is prohibited without express
 *   written permission.
 */

/*
 *  Series of subroutines to convert from parameter types
 *	(byte,int,float,string) to a field and vice versa
 *
 *	This is based on code I received from Ed Bender, Convex Computer Corp.
 *		-- josh
 */


#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <avs/avs.h>
#include <avs/field.h>

/*
 *  int to field
 */
int_parm_to_field()
{
    int		int_parm_to_field_compute();
    int		oport;
    int		parm1;

    AVSset_module_name("int parm to field", MODULE_FILTER);
    oport	= AVScreate_output_port("output field",
				  "field integer 1D scalar uniform 1-space");
    AVSautofree_output(oport);

    parm1	= AVSadd_parameter("integer parameter","integer",
				   1, INT_UNBOUND, INT_UNBOUND);
    AVSconnect_widget(parm1,"typein_integer");

    AVSset_compute_proc(int_parm_to_field_compute);
}

int_parm_to_field_compute(out_field,iparm)
    AVSfield_int **out_field;
    int iparm;
{
  int *out_data;
  int dim[1];

  /*	Build a 1 dimensional array with 1 element			*/

  dim[0]	= 1;
  *out_field	= (AVSfield_int *)
      AVSdata_alloc( "field 1D scalar integer uniform 1-space", dim);
  (*out_field)->data[0]	= iparm;
  return 1;
}


/*
 * float to field
 */
float_parm_to_field()
{
    int		float_parm_to_field_compute();
    int		oport;
    int		parm1;

    AVSset_module_name("float parm to field", MODULE_FILTER);
    oport	= AVScreate_output_port("output field",
				  "field real 1D scalar uniform 1-space");
    AVSautofree_output(oport);

    parm1	= AVSadd_float_parameter("float parameter",
					 1.0, FLOAT_UNBOUND, FLOAT_UNBOUND);
    AVSconnect_widget(parm1,"typein_real");

    AVSset_compute_proc(float_parm_to_field_compute);
}

float_parm_to_field_compute(out_field,fparm)
    AVSfield_float	**out_field;
    float		*fparm;
{
    float		*out_data;
    int			dim[1];

    /*	Build a 1 dimensional array with 1 element			*/

    dim[0]	= 1;
    *out_field	= (AVSfield_float *)
	AVSdata_alloc("field 1D scalar real uniform 1-space", dim);
    (*out_field)->data[0]	= *fparm;
    return 1;
}


/*
 *   string to field
 */
string_parm_to_field()
{
    int		string_parm_to_field_compute();
    int		oport;
    int		parm1;

    AVSset_module_name("string parm to field", MODULE_FILTER);
    oport	= AVScreate_output_port("output field",
				  "field byte 1D scalar uniform 1-space");
    AVSautofree_output(oport);

    parm1	= AVSadd_parameter("string parameter","string",
				   NULL, NULL, NULL);
    AVSconnect_widget(parm1,"typein_string");

    AVSset_compute_proc(string_parm_to_field_compute);
}

string_parm_to_field_compute(out_field,sparm)
    AVSfield_char	**out_field;
    char		*sparm;
{
    float		*out_data;
    int			dim[1];

    if( sparm == NULL ) {
	dim[0]	= 1;
	sparm	= "";
    }
    else {
	dim[0]	= strlen(sparm) + 1;
    }

    *out_field	= (AVSfield_char *)
	AVSdata_alloc("field 1D scalar byte uniform 1-space", dim);
    strcpy( (*out_field)->data, sparm );
    return 1;
}

/*
 *  field to integer
 */
field_to_int_parm()
{
    int		field_to_int_parm_compute();
    int		oport;
    int		iport;

    iport	= AVScreate_input_port("input field",
			       "field 1D scalar integer uniform 1-space",
				REQUIRED);
    AVSset_module_name("field to int parm", MODULE_FILTER);
    oport	= AVScreate_output_port("output integer", "integer" );
    AVSautofree_output(oport);

    AVSset_compute_proc(field_to_int_parm_compute);
}

field_to_int_parm_compute( in_field, ioutp )
    AVSfield_int	*in_field;
    int			*ioutp;
{
    /*	Make sure the field isn't empty					*/

    if( in_field == NULL || in_field->dimensions[0] <= 0 )
	return 0;

    if( ioutp != NULL )
	*ioutp	= in_field->data[0];
    return 1;
}

/*
 *  field to float
 */
field_to_float_parm()
{
    int		field_to_float_parm_compute();
    int		oport;
    int		iport;

    iport	= AVScreate_input_port("input field",
			       "field 1D scalar real uniform 1-space",
				REQUIRED);
    AVSset_module_name("field to float parm", MODULE_FILTER);
    oport	= AVScreate_output_port("output float", "real" );
    AVSautofree_output(oport);

    AVSset_compute_proc(field_to_float_parm_compute);
}

field_to_float_parm_compute( in_field, foutp )
    AVSfield_float	*in_field;
    float		**foutp;
{
    /*	Make sure the field isn't empty					*/

    if( in_field == NULL || in_field->dimensions[0] <= 0 )
	return 0;

    if( foutp != NULL ) {
	if( *foutp == NULL )
	    *foutp	= (float *)malloc(sizeof(float));
	**foutp	= in_field->data[0];
    }
    return 1;
}


/*
 *  field to string
 */
field_to_string_parm()
{
    int		field_to_string_parm_compute();
    int		oport;
    int		iport;

    AVSset_module_name("field to string parm", MODULE_FILTER);
    iport	= AVScreate_input_port("input field",
			       "field 1D scalar byte uniform 1-space",
				REQUIRED);	
    oport	= AVScreate_output_port("output string", "string" );
    AVSautofree_output(oport);

    AVSset_compute_proc(field_to_string_parm_compute);
}

field_to_string_parm_compute( in_field, str )
    AVSfield_char	*in_field;
    char		**str;
{
    int		length;

    if( str == NULL ) {
	fprintf(stderr, "field-to-string-parm: str == NULL\n");
	return 0;
    }
    
    /*	Free any previously existing string before we crush the pointer	*/

    if( *str != NULL )
	free( *str );

    if( in_field->dimensions[0] <= 0 ) {
	*str	= strdup("");
    }
    else {
	length	= in_field->dimensions[0];
	*str	= malloc( length + 1);		/* remember NULL */
	strncpy( *str, in_field->data, length );

	/*	ensure null termination		*/
	(*str)[length] = 0;
    }
    return 1;
}


/*
 * initialize the modules contained in this file.
 */

int ((*data_mod_list[])()) = {
    int_parm_to_field,
    float_parm_to_field,
    string_parm_to_field,
    field_to_int_parm,
    field_to_float_parm,
    field_to_string_parm,
};

#define NMODS sizeof(data_mod_list) / sizeof(char *)

AVSinit_modules()
{
        AVSinit_from_module_list(data_mod_list, 6);
/*
        AVSinit_from_module_list(data_mod_list, NMODS);
*/
}


